/**********************************************************************\
**
**                              Copyright (c) by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a licence and may be used and
**      copied only in accordance with the terms  of  such  licence  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      C56SdkDemo11DlgMain.cpp
**  Facility:    S684002 - Hengstler C56 Printer Driver for Windows XP/7/8/10
**  Component:   C56 SDK Demo Application 10 (C56SdkDemo11)
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2010
**  Abstract:    Implementation of main-dialog of SDK C-56 Demo 11
**  Notes:       -
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "stdafx.h"

#include <winspool.h>

#include "..\..\Libraries\C56SDK.h"  /* API */

#include "C56SdkDemo11.h"
#include "C56SdkDemo11DlgMain.h"

/*
** COMPILER
*/

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/*
** MACROS
*/ 

//-- Macros for writing/reading data to through printer I/O
#define C56SDK_PIOPRINTERWRITE_STRING(hHandle,pszStr,dwTimeOutMs)     C56Sdk_PioPrinterWrite(hHandle, (unsigned char *)pszStr, (DWORD) strlen(pszStr), dwTimeOutMs)
#define C56SDK_PIOPRINTERWRITE_BYTEARRAY(hHandle, aubDat,dwTimeOutMs)  C56Sdk_PioPrinterWrite(hHandle, (unsigned char *)aubDat, (DWORD) sizeof(aubDat), dwTimeOutMs)
#define C56SDK_PIOPRINTERREAD_BYTEARRAY(hHandle, aubDat,dwTimeOutMs)   C56Sdk_PioPrinterRead(hHandle,  (unsigned char *)aubDat, (DWORD) sizeof(aubDat), dwTimeOutMs)

/*
** MESSAGE MAP
*/

BEGIN_MESSAGE_MAP(CC56SdkDemo11DlgMain, CDialog)
   ON_WM_SYSCOMMAND()
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   ON_WM_TIMER()
   //}}AFX_MSG_MAP
   ON_BN_CLICKED(IDC_BUTTON_RESET_COUNTERS,           OnBnClickedButtonResetCounters)
   ON_BN_CLICKED(IDC_BUTTON_INQUIRE_PRINTER_STATUS,   OnBnClickedButtonInquirePrinterStatus)

END_MESSAGE_MAP()

//******************************************************************************************
// METHOD:   CC56SdkDemo11DlgMain::CC56SdkDemo11DlgMain
// ABSTRACT: Object constructor
//******************************************************************************************

CC56SdkDemo11DlgMain::CC56SdkDemo11DlgMain
   (CWnd* pParent /*=NULL*/)
:  CDialog(CC56SdkDemo11DlgMain::IDD, pParent)
{
   //-- Set member icon
   m_hIcon = AfxGetApp()->LoadIcon(IDR_C56);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo11DlgMain::DoDataExchange
// ABSTRACT: Called by the framework to exchange and validate dialog data
//******************************************************************************************

void CC56SdkDemo11DlgMain::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo11DlgMain::OnInitDialog
// ABSTRACT: This member function is called in response to the WM_INITDIALOG message.
//******************************************************************************************

BOOL CC56SdkDemo11DlgMain::OnInitDialog
   ()
{
   //-- Initialize dialog
   CDialog::OnInitDialog();
 
   //-- Initialize member variables 
   m_strPrinter                          = "";
   m_puiPrinterStatusInquiryTimer        = 0;
   m_dwPrinterStatusInquiryCounter       = 0;
   m_dwPrinterStatusInquiryFaults        = 0;
   m_dwPrinterStatusInquiryIntervalMs    = 1000;
   m_fPrinterStatusInquiryEnabled        = FALSE;
   m_fPrinterStatusDwordValid            = false;
   m_aubPrinterStatusDword[0]            = 0;
   m_aubPrinterStatusDword[1]            = 0;
   m_aubPrinterStatusDword[2]            = 0;
   m_aubPrinterStatusDword[3]            = 0;

   //-- Load these dialog items
   this->LoadDlgItems();

   //-- Set C-56 icon for dialog 
   SetIcon(m_hIcon, TRUE);   // Use large symbol verwenden
   SetIcon(m_hIcon, FALSE);  // Use small symbol 

   //-- Setup printer selection combo box with available C-56 printers
   //.. Start access SDK to go through all C-56 printers 
   HANDLE hEnumPrinter; // Handle for enumeration 
   if ((hEnumPrinter = C56Sdk_PmStartEnumPrinter()) != NULL) {

      LPSTR       lpszPrinter; // Name of printer
      CComboBox * pPrinterList;  // Pointer to combo box item

      //-- Get printer selection combo box item
      if (pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) {

         //-- Clear printer selection combo box
         pPrinterList->ResetContent();

         //-- Go through all C-56 printers
         for ( lpszPrinter = C56Sdk_PmEnumPrinter(hEnumPrinter, TRUE)
             ; lpszPrinter != NULL
             ; lpszPrinter = C56Sdk_PmEnumPrinter(hEnumPrinter, FALSE)
             ) {

            //-- Add printer to printer selection combo box 
            pPrinterList->AddString(lpszPrinter);
         }

         //-- Select 1st printer in printer selection combo box
         pPrinterList->SetCurSel(0);
      }
       
      //-- End of enumeration, free handle
      C56Sdk_PmEndEnumPrinter(hEnumPrinter);
   }
      
   //-- Set and update dialog items
   this->SetDlgItems();
   this->UpdateDlgItems();

   //-- Create a timer for update of the printer status inquiry 
   m_puiPrinterStatusInquiryTimer = this->SetTimer(1, m_fPrinterStatusInquiryEnabled ? m_dwPrinterStatusInquiryIntervalMs : 1000, 0);

   //-- Initialization successfull, set focus to first item
   return TRUE;  
}

/******************************************************************************************\
** CLASS  CC56SdkDemo11DlgMain
** METHOD LoadDlgItems
\******************************************************************************************/

void CC56SdkDemo11DlgMain::LoadDlgItems
   ()
{
   //-- Get dialog item settings from ini file
   m_strPrinter                        = theApp.GetProfileString("CC56SdkDemo11DlgMain", "PrinterDriver",   "");
   m_dwPrinterStatusInquiryIntervalMs  = theApp.GetProfileInt   ("CC56SdkDemo11DlgMain", "PrinterStatusInquiryIntervalMs",    1000);
}

/******************************************************************************************\
** CLASS  CC56SdkDemo11DlgMain
** METHOD StoreDlgItems
\******************************************************************************************/

void CC56SdkDemo11DlgMain::StoreDlgItems
   ()
{
   //-- Write dialog item settings in ini file
   theApp.WriteProfileString("CC56SdkDemo11DlgMain", "PrinterDriver",                     m_strPrinter);
   theApp.WriteProfileInt   ("CC56SdkDemo11DlgMain", "PrinterStatusInquiryIntervalMs",    m_dwPrinterStatusInquiryIntervalMs);
}

/******************************************************************************************\
** CLASS  CC56SdkDemo11DlgMain
** METHOD SetDlgItems
\******************************************************************************************/

void CC56SdkDemo11DlgMain::SetDlgItems
   ()
{
   CComboBox * pDlgItemComboBox;      // Pointer to combo box dialog item
   CEdit     * pDlgItemEdit;          // Pointer to edit dialog item
   CButton   * pDlgItemButton;        // Pointer to button dialog item
   
   //-- Get printer selection combo box item
   if (pDlgItemComboBox = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) {
      
      //-- Select driver in combo box
      int iSel; // Selected item
      if ((iSel = pDlgItemComboBox->FindStringExact(0, m_strPrinter)) != CB_ERR)
         pDlgItemComboBox->SetCurSel(iSel);

      //-- Enable/disable window
      pDlgItemComboBox->EnableWindow(!m_fPrinterStatusInquiryEnabled);
   }

   //-- Setup dialog item: printer status inquiry  interval (MS)
   if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_PRINTER_STATUS_INQUIRY_INTERVAL_MS))) != NULL) {

      //-- Setup string with field
      CString strDlgItem;        // String containing dialog item data
      strDlgItem.Format(_T("%d"), m_dwPrinterStatusInquiryIntervalMs);

      //-- Set edit field 
      pDlgItemEdit->SetWindowText(strDlgItem);

      //-- Enable/disable window
      pDlgItemEdit->EnableWindow(!m_fPrinterStatusInquiryEnabled);
   }

   //-- Setup button to start or stop automatic printing
   if ((pDlgItemButton = ((CButton *) this->GetDlgItem(IDC_BUTTON_AUTO_PRINTOUT))) != NULL) {

      //-- Set edit field 
      pDlgItemButton->SetWindowText(m_fPrinterStatusInquiryEnabled ? "Stop Printer Status Inquiry" : "Start Printer Status Inquiry");
   }
}

/******************************************************************************************\
** CLASS  CC56SdkDemo11DlgMain
** METHOD UpdateDlgItems
\******************************************************************************************/

void CC56SdkDemo11DlgMain::UpdateDlgItems
   ()
{
   CStatic   * pDlgItemText;          // Pointer to text dialog item
   CString     strItem;               // Some string

   //-- Update dialog item: status
   if ((pDlgItemText = ((CStatic *) this->GetDlgItem(IDC_STATIC_STATUS))) != NULL) {

      //-- Determine the item text 
      if (m_fPrinterStatusInquiryEnabled) 
           strItem.Format("PRINTER STATUS INQUIRY ENABLED\nNUMBER OF INQUIRIES: %lu\nNUMBER OF FAULTS: %lu", m_dwPrinterStatusInquiryCounter, m_dwPrinterStatusInquiryFaults);
      else strItem.Format("PRINTER STATUS INQUIRY DISABLED");

      //-- Set edit field 
      pDlgItemText->SetWindowText(strItem);

      //-- Enable/disable window
      pDlgItemText->EnableWindow(m_fPrinterStatusInquiryEnabled);
   }

   //-- Update printer status
   if (m_fPrinterStatusInquiryEnabled && m_fPrinterStatusDwordValid) {

      //-- Update printer status bits
      this->GetDlgItem(IDC_RADIO_STATUS_PAPER_IN_CHUTE_DETECTED)->EnableWindow(TRUE);
      this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_IN_CHUTE_DETECTED, (m_aubPrinterStatusDword[0] & C56SR_PRINTER_PAPERINCHUTE) != 0);
      this->GetDlgItem(IDC_RADIO_STATUS_TEMP_ALERT)->EnableWindow(TRUE);
      this->CheckDlgButton(IDC_RADIO_STATUS_TEMP_ALERT, (m_aubPrinterStatusDword[0] & C56SR_PRINTER_TEMPALERT)    != 0);
      this->GetDlgItem(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST)->EnableWindow(TRUE);
      this->CheckDlgButton(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST, (m_aubPrinterStatusDword[0] & C56SR_PRINTER_PRINTOUTLOST) != 0);
      this->GetDlgItem(IDC_RADIO_STATUS_PAPER_JAM)->EnableWindow(TRUE);
      this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_JAM, (m_aubPrinterStatusDword[0] & C56SR_PRINTER_PAPERJAM)     != 0);
      this->GetDlgItem(IDC_RADIO_STATUS_PAPER_LOW)->EnableWindow(TRUE);
      this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_LOW, (m_aubPrinterStatusDword[0] & C56SR_PRINTER_PAPERLOW)     != 0);
      this->GetDlgItem(IDC_RADIO_STATUS_PAPER_OUT)->EnableWindow(TRUE);
      this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_OUT, (m_aubPrinterStatusDword[0] & C56SR_PRINTER_OUTOFPAPER)   != 0);
      this->GetDlgItem(IDC_RADIO_STATUS_POWER_ON)->EnableWindow(TRUE);
      this->CheckDlgButton(IDC_RADIO_STATUS_POWER_ON, (m_aubPrinterStatusDword[0] & C56SR_PRINTER_POWERON)       != 0);
                  
      //-- Update temperature
      strItem.Format("%-3.3d", m_aubPrinterStatusDword[2]);
      this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->EnableWindow(TRUE);
      this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->SetWindowText(strItem);
      this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C_TXT)->EnableWindow(TRUE);
      strItem.Format("%-3.3d", (signed int) ((m_aubPrinterStatusDword[2] * 1.8) + 32));
      this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->EnableWindow(TRUE);
      this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->SetWindowText(strItem);
      this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F_TXT)->EnableWindow(TRUE);
                  
      //-- Update printer status hex code
      strItem.Format( "[%2.2X][%2.2X][%2.2X][%2.2X]"
                    , (WORD) (((BYTE *) &m_aubPrinterStatusDword)[0])
                    , (WORD) (((BYTE *) &m_aubPrinterStatusDword)[1])
                    , (WORD) (((BYTE *) &m_aubPrinterStatusDword)[2])
                    , (WORD) (((BYTE *) &m_aubPrinterStatusDword)[3]));
      this->GetDlgItem(IDC_STATIC_STATUS_HEX)->EnableWindow(TRUE);
      this->GetDlgItem(IDC_STATIC_STATUS_HEX)->SetWindowText(strItem);
   } 
   else {
   
       //--Invalidate printer status bits
       this->GetDlgItem(IDC_RADIO_STATUS_PAPER_IN_CHUTE_DETECTED)->EnableWindow(FALSE);
       this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_IN_CHUTE_DETECTED, FALSE);
       this->GetDlgItem(IDC_RADIO_STATUS_TEMP_ALERT)->EnableWindow(FALSE);
       this->CheckDlgButton(IDC_RADIO_STATUS_TEMP_ALERT, FALSE);
       this->GetDlgItem(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST)->EnableWindow(FALSE);
       this->CheckDlgButton(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST, FALSE);
       this->GetDlgItem(IDC_RADIO_STATUS_PAPER_JAM)->EnableWindow(FALSE);
       this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_JAM, FALSE);
       this->GetDlgItem(IDC_RADIO_STATUS_PAPER_LOW)->EnableWindow(FALSE);
       this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_LOW, FALSE);
       this->GetDlgItem(IDC_RADIO_STATUS_PAPER_OUT)->EnableWindow(FALSE);
       this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_OUT, FALSE);
       this->GetDlgItem(IDC_RADIO_STATUS_POWER_ON)->EnableWindow(FALSE);
       this->CheckDlgButton(IDC_RADIO_STATUS_POWER_ON, FALSE);

       //-- Invalidate printer temperature
       this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->EnableWindow(FALSE);
       this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->SetWindowText("-");
       this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C_TXT)->EnableWindow(FALSE);
       this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->EnableWindow(FALSE);
       this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->SetWindowText("-");
       this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F_TXT)->EnableWindow(FALSE);

       //-- Invalidate printer hex code
       this->GetDlgItem(IDC_STATIC_STATUS_HEX)->EnableWindow(FALSE);
       this->GetDlgItem(IDC_STATIC_STATUS_HEX)->SetWindowText("[??][??][??][??]");
   }            
}

/******************************************************************************************\
** CLASS  CC56SdkDemo11DlgMain
** METHOD GetDlgItems
\******************************************************************************************/

void CC56SdkDemo11DlgMain::GetDlgItems
   ()
{
   CComboBox * pDlgItemComboBox;      // Pointer to combo box dialog item
   CEdit     * pDlgItemEdit;          // Pointer to edit dialog item
 
   //-- Get dialog item: printer driver 
   m_strPrinter = "";
   if (pDlgItemComboBox = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) {

      //-- Get actual selected driver 
      INT nIndex;
      if ((nIndex = pDlgItemComboBox->GetCurSel()) != LB_ERR)
         pDlgItemComboBox->GetLBText(nIndex, m_strPrinter);
   }

   //-- Get dialog item: printer status inquiry interval (MS)
   if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_PRINTER_STATUS_INQUIRY_INTERVAL_MS))) != NULL) {

      //-- Get edit field contents
      CString strDlgItem;        // String containing dialog item data
      pDlgItemEdit->GetWindowText(strDlgItem);

      //-- Set print interval 
      m_dwPrinterStatusInquiryIntervalMs = abs(atoi(strDlgItem.GetBuffer())); 
      strDlgItem.ReleaseBuffer();
   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo11DlgMain::OnSysCommand
// ABSTRACT: The framework calls this member function when the user selects a command from 
//           the Control menu, or when the user selects the Maximize or the Minimize button.
//******************************************************************************************

void CC56SdkDemo11DlgMain::OnSysCommand
   (UINT nID, LPARAM lParam)
{
    //-- Store dialog items on close 
   if (nID == SC_CLOSE) {

      //-- Get dialog item settings 
      this->GetDlgItems();

      //-- Store these dialog items
      this->StoreDlgItems();
   }

   //-- Call original CDialog::OnSysCommand member function
   CDialog::OnSysCommand(nID, lParam);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo11DlgMain::OnPaint
// ABSTRACT: The framework calls this member function when Windows or an application makes a 
//           request to repaint a portion of an application's window.
//******************************************************************************************

void CC56SdkDemo11DlgMain::OnPaint
   () 
{
   //-- Check if dialog is iconic
   if (IsIconic()) {

      CPaintDC dc(this); // Device context for paint 

      //-- Erase background of icon
      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      //-- Center symbol in clientr rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      //-- Draw symbol 
      dc.DrawIcon(x, y, m_hIcon);
   }
   else {

      //-- Call original CDialog::OnPaint member function
      CDialog::OnPaint();
   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo11DlgMain::OnQueryDragIcon
// ABSTRACT: The framework calls this member function by a minimized (iconic) window that 
//           does not have an icon defined for its class.
//******************************************************************************************

HCURSOR CC56SdkDemo11DlgMain::OnQueryDragIcon
   ()
{
   return static_cast<HCURSOR>(m_hIcon);
}

/******************************************************************************************\
** CLASS  CC56SdkDemo11DlgMain
** METHOD OnBnClickedButtonInquirePrinterStatus
\******************************************************************************************/

void CC56SdkDemo11DlgMain::OnBnClickedButtonInquirePrinterStatus
   ()
{
   //-- Get dialog item settings 
   this->GetDlgItems();

   //-- Toggle printer status inquiry flag
   m_fPrinterStatusInquiryEnabled = !m_fPrinterStatusInquiryEnabled;

   //-- Correct inquiry interval
   m_dwPrinterStatusInquiryIntervalMs = max(m_dwPrinterStatusInquiryIntervalMs, 1);

   //-- Reset printer status DWORD 
   m_fPrinterStatusDwordValid            = false;
   m_aubPrinterStatusDword[0]            = 0;
   m_aubPrinterStatusDword[1]            = 0;
   m_aubPrinterStatusDword[2]            = 0;
   m_aubPrinterStatusDword[3]            = 0;
 
   //-- Store these dialog items
   this->StoreDlgItems();

   //-- Set and update dialog items
   this->SetDlgItems();
   this->UpdateDlgItems();

   //-- Kill timer and create a new one 
   this->KillTimer(1);  
   m_puiPrinterStatusInquiryTimer = this->SetTimer(1, m_fPrinterStatusInquiryEnabled ? m_dwPrinterStatusInquiryIntervalMs : 1000, 0);

   //-- Set focus on start stop button
   this->GetDlgItem(IDC_BUTTON_AUTO_PRINTOUT)->SetFocus();
}

/******************************************************************************************\
** CLASS  CC56SdkDemo11DlgMain
** METHOD OnBnClickedButtonResetCounters
\******************************************************************************************/

void CC56SdkDemo11DlgMain::OnBnClickedButtonResetCounters
   ()
{
   //-- Reset counters
   m_dwPrinterStatusInquiryCounter = 0;
   m_dwPrinterStatusInquiryFaults  = 0;

   //-- Set and update dialog items
   this->SetDlgItems();
   this->UpdateDlgItems();
}

//******************************************************************************************
// METHOD:   CC56SdkDemo11DlgMain::DoUpdateStatus
// ABSTRACT: This member function is to update status information displayed 
//******************************************************************************************

VOID CC56SdkDemo11DlgMain::InquirePrinterStatus
   (VOID)
{

   //-- Check if printer status inquiry is enabled
   if (m_fPrinterStatusInquiryEnabled) {

      //-- By default the printer status inquiry failed
      bool fPrinterStatusInquired = false;

      //-- Increase the printer status inquiry counter
      m_dwPrinterStatusInquiryCounter++;

      //-- Get printer list (dialog item combo box)
      CComboBox * pPrinterList;  // Pointer to combo box item
      if ((pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) != NULL) {

         //-- Determine selected printer
         INT iPrinterSel; // Selected printer
         if ((iPrinterSel = pPrinterList->GetCurSel()) != CB_ERR) {

            //-- Allocate memory for this printer and get name from combo box
            LPSTR lpszPrinter = NULL;  // Printer name
            if (    ((lpszPrinter = (LPSTR) malloc(pPrinterList->GetLBTextLen(iPrinterSel)+1)) != NULL) 
                 && (pPrinterList->GetLBText(iPrinterSel, lpszPrinter) != CB_ERR)
               ) {

               //-- Open connection to printer
               HANDLE        hPrinterIo;                                         // Handle for printer I/O
               if ((hPrinterIo = C56Sdk_PioPrinterOpen(lpszPrinter, 250)) != NULL) {
            
                   //-- Transmit the parameter byte
                  //.. and request the "printer status DWORD"
                  //.. and receive the "printer status DWORD"
                  unsigned char aubCmdSetParameterByte[] = {0x1C, 0x72, (BYTE) (m_dwPrinterStatusInquiryCounter & 0xFF)};  // command to "set parameter)
                  unsigned char aubCmdGetPrinterStatus[] = {0x1D, 0x61, 0x01};                                             // command to "get printer status"
                  if (    C56Sdk_PioPrinterWrite(hPrinterIo, aubCmdSetParameterByte, sizeof(aubCmdSetParameterByte),  500) 
                       && C56Sdk_PioPrinterWrite(hPrinterIo, aubCmdGetPrinterStatus, sizeof(aubCmdGetPrinterStatus),  500) 
                       && C56Sdk_PioPrinterRead(hPrinterIo, m_aubPrinterStatusDword, sizeof(m_aubPrinterStatusDword), 1000)
                     ) {
                  
                     //-- Raise "printer status inquired" flag
                     fPrinterStatusInquired = true;
                  }
                   
                  //-- Close printer pipe-connection            
                  C56Sdk_PioPrinterClose(hPrinterIo);   
               }  
         
               //-- Free allocated memory
               if (lpszPrinter) free(lpszPrinter); lpszPrinter = NULL; 
            }  
         }    
      } 

      //-- Check if  the "printer status was inquired"
      if (fPrinterStatusInquired) {

         //-- Raise "printer status DWORD" valid flag
         m_fPrinterStatusDwordValid = true;
      }  
      //.. Could not request or did not receive the "printer status DWORD"
      else {

         //-- Lower "printer status DWORD" valid flag
         m_fPrinterStatusDwordValid = false;
 
         //-- Increase the printer status inquiry faults
         m_dwPrinterStatusInquiryFaults++;
      }
   }
   //.. Printer status inquiry is disabled
   else {

      //-- Lower "printer status DWORD" valid flag
      m_fPrinterStatusDwordValid = false;
   }

   //-- Update dialog items
   this->UpdateDlgItems();
}

//******************************************************************************************
// METHOD:   CC56SdkDemo11DlgMain::OnTimer
// ABSTRACT: This member function is called in response to the WM_TIMER message.
//******************************************************************************************

VOID CC56SdkDemo11DlgMain::OnTimer
   (UINT_PTR nIdEvent)
{
   //-- Inquire printer status
   if (nIdEvent == m_puiPrinterStatusInquiryTimer) {
  
      //-- Inquire printer status if enabled
      if (m_fPrinterStatusInquiryEnabled)
         this->InquirePrinterStatus();

      //-- Update dialog items
      this->UpdateDlgItems();
   } 
   
   else {
      //-- Other dialog timer
      CDialog::OnTimer(nIdEvent);
   }
}
